@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2004-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -188,6 +188,12 @@ static void
 chatrooms_window_destroy_cb (GtkWidget             *widget,
 			     EmpathyChatroomsWindow *window)
 {
+	g_signal_handlers_disconnect_by_func (window->manager,
+					      chatrooms_window_chatroom_added_cb,
+					      window);
+	g_signal_handlers_disconnect_by_func (window->manager,
+					      chatrooms_window_chatroom_removed_cb,
+					      window);
 	g_object_unref (window->manager);
 	g_free (window);
 }
