@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2002-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,7 +40,6 @@
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-debug.h>
-#include <libempathy/empathy-conf.h>
 
 #include "empathy-group-chat.h"
 #include "empathy-chat.h"
@@ -51,6 +50,7 @@
 //#include "empathy-sound.h"
 #include "empathy-images.h"
 #include "empathy-ui-utils.h"
+#include "empathy-conf.h"
 #include "empathy-preferences.h"
 
 #define DEBUG_DOMAIN "GroupChat"
@@ -355,20 +355,18 @@ group_chat_members_changed_cb (EmpathyTpChatroom *tp_chat,
 			       gboolean            is_member,
 			       EmpathyGroupChat   *chat)
 {
-	EmpathyGroupChatPriv *priv;
-	gchar                *str;
-
-	priv = GET_PRIV (chat);
-
-	if (is_member) {
-		str = g_strdup_printf (_("%s has joined the room"),
-				       empathy_contact_get_name (contact));
-	} else {
-		str = g_strdup_printf (_("%s has left the room"),
-				       empathy_contact_get_name (contact));
+	if (!EMPATHY_CHAT (chat)->block_events) {
+		gchar *str;
+		if (is_member) {
+			str = g_strdup_printf (_("%s has joined the room"),
+					       empathy_contact_get_name (contact));
+		} else {
+			str = g_strdup_printf (_("%s has left the room"),
+					       empathy_contact_get_name (contact));
+		}
+		empathy_chat_view_append_event (EMPATHY_CHAT (chat)->view, str);
+		g_free (str);
 	}
-	empathy_chat_view_append_event (EMPATHY_CHAT (chat)->view, str);
-	g_free (str);
 }
 
 static void
@@ -527,8 +525,14 @@ group_chat_set_tp_chat (EmpathyChat    *chat,
 
 	/* Create contact list */
 	priv->store = empathy_contact_list_store_new (EMPATHY_CONTACT_LIST (priv->tp_chat));
-	priv->view = empathy_contact_list_view_new (priv->store);
-	empathy_contact_list_view_set_interactive (priv->view, TRUE);
+	priv->view = empathy_contact_list_view_new (priv->store,
+						    EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CHAT |
+						    EMPATHY_CONTACT_LIST_FEATURE_CONTACT_CALL |
+						    EMPATHY_CONTACT_LIST_FEATURE_CONTACT_LOG |
+						    EMPATHY_CONTACT_LIST_FEATURE_CONTACT_FT |
+						    EMPATHY_CONTACT_LIST_FEATURE_CONTACT_INVITE |
+						    EMPATHY_CONTACT_LIST_FEATURE_CONTACT_INFO);
+
 	gtk_container_add (GTK_CONTAINER (priv->scrolled_window_contacts),
 			   GTK_WIDGET (priv->view));
 	gtk_widget_show (GTK_WIDGET (priv->view));
@@ -565,13 +569,15 @@ group_chat_subject_notify_cb (EmpathyTpChat   *tp_chat,
 	priv->topic = str;
 	gtk_label_set_text (GTK_LABEL (priv->label_topic), priv->topic);
 
-	if (!G_STR_EMPTY (priv->topic)) {
-		str = g_strdup_printf (_("Topic set to: %s"), priv->topic);
-	} else {
-		str = g_strdup (_("No topic defined"));
+	if (!EMPATHY_CHAT (chat)->block_events) {
+		if (!G_STR_EMPTY (priv->topic)) {
+			str = g_strdup_printf (_("Topic set to: %s"), priv->topic);
+		} else {
+			str = g_strdup (_("No topic defined"));
+		}
+		empathy_chat_view_append_event (EMPATHY_CHAT (chat)->view, str);
+		g_free (str);
 	}
-	empathy_chat_view_append_event (EMPATHY_CHAT (chat)->view, str);
-	g_free (str);
 }
 
 static void
