@@ -16,6 +16,9 @@ GType empathy_contact_widget_flags_get_type (void);
 GType empathy_contact_list_store_sort_get_type (void);
 #define EMPATHY_TYPE_CONTACT_LIST_STORE_COL empathy_contact_list_store_col_get_type()
 GType empathy_contact_list_store_col_get_type (void);
+#include <libempathy-gtk/empathy-contact-list-view.h>
+#define EMPATHY_TYPE_CONTACT_LIST_FEATURES empathy_contact_list_features_get_type()
+GType empathy_contact_list_features_get_type (void);
 #include <libempathy-gtk/empathy-chat-view.h>
 #define EMPATHY_TYPE_CHAT_VIEW_BLOCK empathy_chat_view_block_get_type()
 GType empathy_chat_view_block_get_type (void);
