@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2002-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -187,7 +187,7 @@ empathy_main_window_show (void)
 	gint                      x, y, w, h;
 
 	if (window) {
-		gtk_window_present (GTK_WINDOW (window->window));
+		empathy_window_present (GTK_WINDOW (window->window), TRUE);
 		return window->window;
 	}
 
