@@ -30,13 +30,13 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
-#include <libempathy/empathy-conf.h>
-
+#include "empathy-conf.h"
 #include "empathy-preferences.h"
 #include "empathy-ui-utils.h"
 #include "empathy-theme-manager.h"
 #include "empathy-spell.h"
 #include "empathy-contact-list-store.h"
+#include "empathy-gtk-enum-types.h"
 
 typedef struct {
 	GtkWidget *dialog;
@@ -49,6 +49,7 @@ typedef struct {
 	GtkWidget *combobox_chat_theme;
 	GtkWidget *checkbutton_theme_chat_room;
 	GtkWidget *checkbutton_separate_chat_windows;
+	GtkWidget *checkbutton_autoconnect;
 	GtkWidget *radiobutton_contact_list_sort_by_name;
 	GtkWidget *radiobutton_contact_list_sort_by_state;
 
@@ -58,7 +59,6 @@ typedef struct {
 	GtkWidget *checkbutton_popups_when_available;
 
 	GtkWidget *treeview_spell_checker;
-	GtkWidget *checkbutton_spell_checker;
 
 	GList     *notify_ids;
 } EmpathyPreferences;
@@ -197,16 +197,13 @@ preferences_setup_widgets (EmpathyPreferences *preferences)
 					  EMPATHY_PREFS_CHAT_THEME_CHAT_ROOM,
 					  preferences->checkbutton_theme_chat_room);
 
-	preferences_hookup_toggle_button (preferences,
-					  EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
-					  preferences->checkbutton_spell_checker);
-	preferences_hookup_sensitivity (preferences,
-					EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
-					preferences->treeview_spell_checker);
-
 	preferences_hookup_radio_button (preferences,
 					 EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM,
 					 preferences->radiobutton_contact_list_sort_by_name);
+
+	preferences_hookup_toggle_button (preferences,
+					  EMPATHY_PREFS_AUTOCONNECT,
+					  preferences->checkbutton_autoconnect);
 }
 
 static void
@@ -274,6 +271,14 @@ preferences_languages_add (EmpathyPreferences *preferences)
 	store = GTK_LIST_STORE (gtk_tree_view_get_model (view));
 
 	codes = empathy_spell_get_language_codes ();
+
+	empathy_conf_set_bool (empathy_conf_get(),
+			       EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+			       codes != NULL);
+	if (!codes) {
+		gtk_widget_set_sensitive (preferences->treeview_spell_checker, FALSE);
+	}		
+
 	for (l = codes; l; l = l->next) {
 		GtkTreeIter  iter;
 		const gchar *code;
@@ -310,14 +315,14 @@ preferences_languages_save (EmpathyPreferences *preferences)
 				(GtkTreeModelForeachFunc) preferences_languages_save_foreach,
 				&languages);
 
-	if (!languages) {
-		/* Default to english */
-		languages = g_strdup ("en");
-	}
+	/* if user selects no languages, we don't want spell check */
+	empathy_conf_set_bool (empathy_conf_get (),
+			       EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+			       languages != NULL);
 
 	empathy_conf_set_string (empathy_conf_get (),
 				 EMPATHY_PREFS_CHAT_SPELL_CHECKER_LANGUAGES,
-				 languages);
+				 languages ? languages : "");
 
 	g_free (languages);
 }
@@ -938,6 +943,7 @@ empathy_preferences_show (GtkWindow *parent)
 		"combobox_chat_theme", &preferences->combobox_chat_theme,
 		"checkbutton_theme_chat_room", &preferences->checkbutton_theme_chat_room,
 		"checkbutton_separate_chat_windows", &preferences->checkbutton_separate_chat_windows,
+		"checkbutton_autoconnect", &preferences->checkbutton_autoconnect,
 		"radiobutton_contact_list_sort_by_name", &preferences->radiobutton_contact_list_sort_by_name,
 		"radiobutton_contact_list_sort_by_state", &preferences->radiobutton_contact_list_sort_by_state,
 		"checkbutton_sounds_for_messages", &preferences->checkbutton_sounds_for_messages,
@@ -945,7 +951,6 @@ empathy_preferences_show (GtkWindow *parent)
 		"checkbutton_sounds_when_away", &preferences->checkbutton_sounds_when_away,
 		"checkbutton_popups_when_available", &preferences->checkbutton_popups_when_available,
 		"treeview_spell_checker", &preferences->treeview_spell_checker,
-		"checkbutton_spell_checker", &preferences->checkbutton_spell_checker,
 		NULL);
 
 	empathy_glade_connect (glade,
