@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2002-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -223,7 +223,7 @@ private_chat_contact_presence_updated_cb (EmpathyContact     *contact,
 		      empathy_contact_get_id (contact));
 
 	if (!empathy_contact_is_online (contact)) {
-		if (priv->is_online) {
+		if (priv->is_online && !EMPATHY_CHAT (chat)->block_events) {
 			gchar *msg;
 
 			msg = g_strdup_printf (_("%s went offline"),
@@ -237,7 +237,7 @@ private_chat_contact_presence_updated_cb (EmpathyContact     *contact,
 		g_signal_emit_by_name (chat, "composing", FALSE);
 
 	} else {
-		if (!priv->is_online) {
+		if (!priv->is_online && !EMPATHY_CHAT (chat)->block_events) {
 			gchar *msg;
 
 			msg = g_strdup_printf (_("%s has come online"),
