@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2004 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -76,8 +76,6 @@ enum {
 	PROP_ACCOUNT,
 	PROP_PRESENCE,
 	PROP_PRESENCE_MESSAGE,
-	PROP_GROUPS,
-	PROP_SUBSCRIPTION,
 	PROP_HANDLE,
 	PROP_CAPABILITIES,
 	PROP_IS_USER
@@ -159,8 +157,8 @@ empathy_contact_class_init (EmpathyContactClass *class)
 							     "Contact Capabilities",
 							     "Capabilities of the contact",
 							     EMPATHY_TYPE_CAPABILITIES,
-							     0,
-							     G_PARAM_READWRITE));
+							     EMPATHY_CAPABILITIES_UNKNOWN,
+							     G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_IS_USER,
