@@ -1,21 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
@@ -183,7 +182,7 @@ empathy_tp_roomlist_new (McAccount *account)
 
 	priv->account = g_object_ref (account);
 	priv->roomlist_iface = tp_chan_get_interface (priv->tp_chan,
-						      TELEPATHY_CHAN_IFACE_ROOMLIST_QUARK);
+						      TP_IFACE_QUARK_CHANNEL_TYPE_ROOM_LIST);
 
 	g_signal_connect (priv->tp_chan, "destroy",
 			  G_CALLBACK (tp_roomlist_destroy_cb),
@@ -318,6 +317,7 @@ tp_roomlist_got_rooms_cb (DBusGProxy        *roomlist_iface,
 		EmpathyChatroom *chatroom;
 		gchar           *room_id;
 		const gchar     *room_name;
+		const GValue    *room_name_value;
 		GValueArray     *room_struct;
 		guint            handle;
 		const gchar     *channel_type;
@@ -330,7 +330,8 @@ tp_roomlist_got_rooms_cb (DBusGProxy        *roomlist_iface,
 		info = g_value_get_boxed (g_value_array_get_nth (room_struct, 2));
 
 		/* Create the chatroom */
-		room_name = g_hash_table_lookup (info, "name");
+		room_name_value = g_hash_table_lookup (info, "name");
+		room_name = g_value_get_string (room_name_value);
 		room_id = empathy_inspect_handle (priv->account,
 						  handle,
 						  TP_HANDLE_TYPE_ROOM);
