@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2003-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,9 +34,11 @@
 
 #include <libxml/uri.h>
 #include <libtelepathy/tp-helpers.h>
+#include <libtelepathy/tp-conn.h>
 
 #include "empathy-debug.h"
 #include "empathy-utils.h"
+#include "empathy-contact-factory.h"
 #include "empathy-contact-manager.h"
 #include "empathy-tp-group.h"
 
@@ -57,7 +59,7 @@ empathy_substring (const gchar *str,
  */
 #define USERCHARS "-A-Za-z0-9"
 #define PASSCHARS "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
-#define HOSTCHARS "-A-Za-z0-9"
+#define HOSTCHARS "-A-Za-z0-9_"
 #define PATHCHARS "-A-Za-z0-9_$.+!*(),;:@&=?/~#%"
 #define SCHEME    "(news:|telnet:|nntp:|file:/|https?:|ftps?:|webcal:)"
 #define USER      "[" USERCHARS "]+(:["PASSCHARS "]+)?"
@@ -208,11 +210,16 @@ empathy_xml_validate (xmlDoc      *doc,
 	xmlDtd       *dtd;
 	gboolean      ret;
 
-	path = g_build_filename (DATADIR, "empathy", dtd_filename, NULL);
+	path = g_build_filename (g_getenv ("EMPATHY_SRCDIR"), "libempathy",
+				 dtd_filename, NULL);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+		path = g_build_filename (DATADIR, "empathy", dtd_filename, NULL);
+	}
+	empathy_debug (DEBUG_DOMAIN, "Loading dtd file %s", path);
 
 	/* The list of valid chars is taken from libxml. */
 	escaped = xmlURIEscapeStr (path, ":@&=+$,/?;");
-
 	g_free (path);
 
 	memset (&cvp, 0, sizeof (cvp));
@@ -393,17 +400,19 @@ empathy_inspect_handle (McAccount *account,
 }
 
 void
-empathy_call_contact (EmpathyContact *contact)
+empathy_call_with_contact (EmpathyContact *contact)
 {
 #ifdef HAVE_VOIP
-	MissionControl *mc;
-	McAccount      *account;
-	TpConn         *tp_conn;
-	gchar          *object_path;
-	const gchar    *bus_name;
-	TpChan         *new_chan;
-	EmpathyTpGroup *group;
-	GError         *error = NULL;
+	MissionControl        *mc;
+	McAccount             *account;
+	TpConn                *tp_conn;
+	gchar                 *object_path;
+	const gchar           *bus_name;
+	TpChan                *new_chan;
+	EmpathyContactFactory *factory;
+	EmpathyTpGroup        *group;
+	EmpathyContact        *self_contact;
+	GError                *error = NULL;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
@@ -441,8 +450,13 @@ empathy_call_contact (EmpathyContact *contact)
 				0);
 
 	group = empathy_tp_group_new (account, new_chan);
+	factory = empathy_contact_factory_new ();
+	self_contact = empathy_contact_factory_get_user (factory, account);
 	empathy_tp_group_add_member (group, contact, "");
+	empathy_tp_group_add_member (group, self_contact, "");	
 
+	g_object_unref (factory);
+	g_object_unref (self_contact);
 	g_object_unref (group);
 	g_object_unref (mc);
 	g_object_unref (tp_conn);
@@ -451,3 +465,141 @@ empathy_call_contact (EmpathyContact *contact)
 #endif
 }
 
+#ifdef HAVE_VOIP
+static void
+got_handle_cb (EmpathyContact        *contact, 
+	       GParamSpec            *property,
+	       EmpathyContactFactory *factory)
+{
+	g_signal_handlers_disconnect_by_func (contact,
+					      got_handle_cb,
+					      factory);
+
+	empathy_call_with_contact (contact);
+	g_object_unref (factory);
+	g_object_unref (contact);
+}
+#endif
+
+void
+empathy_call_with_contact_id (McAccount *account, const gchar *contact_id)
+{
+#ifdef HAVE_VOIP
+	EmpathyContactFactory *factory;
+	EmpathyContact        *contact;
+
+	factory = empathy_contact_factory_new ();
+	contact = empathy_contact_factory_get_from_id (factory, account, contact_id);
+
+	if (empathy_contact_get_handle (contact) != 0) {
+		empathy_call_with_contact (contact);
+		g_object_unref (contact);
+		g_object_unref (factory);
+	} else {
+		g_signal_connect (contact, "notify::handle",
+				  G_CALLBACK (got_handle_cb),
+				  factory);
+	}
+#endif
+}
+
+void
+empathy_chat_with_contact (EmpathyContact  *contact)
+{
+	MissionControl *mc;
+
+	mc = empathy_mission_control_new ();
+	mission_control_request_channel (mc,
+					 empathy_contact_get_account (contact),
+					 TP_IFACE_CHANNEL_TYPE_TEXT,
+					 empathy_contact_get_handle (contact),
+					 TP_HANDLE_TYPE_CONTACT,
+					 NULL, NULL);
+	g_object_unref (mc);
+}
+
+void
+empathy_chat_with_contact_id (McAccount *account, const gchar *contact_id)
+{
+	MissionControl *mc;
+
+	mc = empathy_mission_control_new ();
+	mission_control_request_channel_with_string_handle (mc,
+							    account,
+							    TP_IFACE_CHANNEL_TYPE_TEXT,
+							    contact_id,
+							    TP_HANDLE_TYPE_CONTACT,
+							    NULL, NULL);
+	g_object_unref (mc);
+}
+
+const gchar *
+empathy_presence_get_default_message (McPresence presence)
+{
+	switch (presence) {
+	case MC_PRESENCE_AVAILABLE:
+		return _("Available");
+	case MC_PRESENCE_DO_NOT_DISTURB:
+		return _("Busy");
+	case MC_PRESENCE_AWAY:
+	case MC_PRESENCE_EXTENDED_AWAY:
+		return _("Away");
+	case MC_PRESENCE_HIDDEN:
+		return _("Hidden");
+	case MC_PRESENCE_OFFLINE:
+	case MC_PRESENCE_UNSET:
+		return _("Offline");
+	default:
+		g_assert_not_reached ();
+	}
+
+	return NULL;
+}
+
+const gchar *
+empathy_presence_to_str (McPresence presence)
+{
+	switch (presence) {
+	case MC_PRESENCE_AVAILABLE:
+		return "available";
+	case MC_PRESENCE_DO_NOT_DISTURB:
+		return "busy";
+	case MC_PRESENCE_AWAY:
+		return "away";
+	case MC_PRESENCE_EXTENDED_AWAY:
+		return "ext_away";
+	case MC_PRESENCE_HIDDEN:
+		return "hidden";
+	case MC_PRESENCE_OFFLINE:
+		return "offline";
+	case MC_PRESENCE_UNSET:
+		return "unset";
+	default:
+		g_assert_not_reached ();
+	}
+
+	return NULL;
+}
+
+McPresence
+empathy_presence_from_str (const gchar *str)
+{
+	if (strcmp (str, "available") == 0) {
+		return MC_PRESENCE_AVAILABLE;
+	} else if ((strcmp (str, "dnd") == 0) || (strcmp (str, "busy") == 0)) {
+		return MC_PRESENCE_DO_NOT_DISTURB;
+	} else if ((strcmp (str, "away") == 0) || (strcmp (str, "brb") == 0)) {
+		return MC_PRESENCE_AWAY;
+	} else if ((strcmp (str, "xa") == 0) || (strcmp (str, "ext_away") == 0)) {
+		return MC_PRESENCE_EXTENDED_AWAY;
+	} else if (strcmp (str, "hidden") == 0) {
+		return MC_PRESENCE_HIDDEN;
+	} else if (strcmp (str, "offline") == 0) {
+		return MC_PRESENCE_OFFLINE;
+	} else if (strcmp (str, "unset") == 0) {
+		return MC_PRESENCE_UNSET;
+	}
+
+	return MC_PRESENCE_AVAILABLE;
+}
+
